==> reverie_dreams-0.4.3%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/block/entity/MusicBlockEntity.java <==
package cc.thonly.reverie_dreams.block.entity;

import cc.thonly.reverie_dreams.util.TouhouNotaUtils;
import lombok.Getter;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2586;
import net.minecraft.class_2680;
import nota.player.SongPlayer;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Getter
public class MusicBlockEntity extends class_2586 {
    private String select = null;
    public boolean isFirst = true;

    public MusicBlockEntity(class_2338 pos, class_2680 state) {
        super(ModBlockEntities.MUSIC_BLOCK_ENTITY, pos, state);
    }

    @Nullable
    public SongPlayer getSelfPlayer() {
        Map<Long, SongPlayer> blockPos2SongPlayer = TouhouNotaUtils.blockMusicPlayCache.get(this.field_11863);
        if (blockPos2SongPlayer == null) return null;
        return blockPos2SongPlayer.get(this.field_11867.method_10063());
    }

    public static synchronized void tick(class_1937 world, class_2338 pos, class_2680 state, MusicBlockEntity blockEntity) {
        if (world.method_8608()) return;

        boolean hasRedstone = world.method_49803(pos);

        Map<Long, SongPlayer> blockPos2SongPlayer = TouhouNotaUtils.blockMusicPlayCache.computeIfAbsent(world, w -> new HashMap<>());
        SongPlayer songPlayer = blockPos2SongPlayer.get(pos.method_10063());

        if (hasRedstone && blockEntity.isFirst && blockEntity.select != null) {
            blockEntity.isFirst = false;
            TouhouNotaUtils.playAt(world, pos, blockEntity.getSelect());
            return;
        }

        if (hasRedstone && songPlayer == null) {
            TouhouNotaUtils.playAt(world, pos, blockEntity.getSelect());
            return;
        }

        if (!hasRedstone && songPlayer != null && songPlayer.isPlaying()) {
            songPlayer.setPlaying(false);
            blockPos2SongPlayer.remove(pos.method_10063());
        }
    }

    @Override
    protected void method_11007(class_11372 view) {
        super.method_11007(view);
        view.method_71469("Select", this.select == null ? "" : this.select);
    }

    @Override
    protected void method_11014(class_11368 view) {
        super.method_11014(view);
        this.select = view.method_71428("Select","");
    }

    public int play() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty() || select == null || !filenames.contains(select)) {
            return -1;
        }

        if (this.field_11863 != null && !this.field_11863.field_9236) {
            TouhouNotaUtils.playAt(field_11863, field_11867, select);
        }
        return filenames.indexOf(select);
    }

    public int prev() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty()) return -1;

        int index = select == null ? 0 : filenames.indexOf(select);
        if (index == -1) index = 0;

        index = (index - 1 + filenames.size()) % filenames.size();
        this.select = filenames.get(index);
        this.method_5431();
        return index;
    }

    public int next() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty()) return -1;

        int index = select == null ? -1 : filenames.indexOf(select);
        index = (index + 1) % filenames.size();
        this.select = filenames.get(index);
        this.method_5431();
        return index;
    }


    public List<String> getFilenames() {
        return TouhouNotaUtils.getFileNames();
    }
}

==> reverie_dreams-0.4.6%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/block/entity/MusicBlockEntity.java <==
package cc.thonly.reverie_dreams.block.entity;

import cc.thonly.reverie_dreams.util.TouhouNotaUtils;
import lombok.Getter;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2586;
import net.minecraft.class_2680;
import nota.player.SongPlayer;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Getter
public class MusicBlockEntity extends class_2586 {
    private String select = null;
    public boolean isFirst = true;

    public MusicBlockEntity(class_2338 pos, class_2680 state) {
        super(RDBlockEntityTypes.MUSIC_BLOCK_ENTITY, pos, state);
    }

    @Nullable
    public SongPlayer getSelfPlayer() {
        Map<Long, SongPlayer> blockPos2SongPlayer = TouhouNotaUtils.blockMusicPlayCache.get(this.field_11863);
        if (blockPos2SongPlayer == null) return null;
        return blockPos2SongPlayer.get(this.field_11867.method_10063());
    }

    public static synchronized void tick(class_1937 world, class_2338 pos, class_2680 state, MusicBlockEntity blockEntity) {
        if (world.method_8608()) return;

        boolean hasRedstone = world.method_49803(pos);

        Map<Long, SongPlayer> blockPos2SongPlayer = TouhouNotaUtils.blockMusicPlayCache.computeIfAbsent(world, w -> new HashMap<>());
        SongPlayer songPlayer = blockPos2SongPlayer.get(pos.method_10063());

        if (hasRedstone && blockEntity.isFirst && blockEntity.select != null) {
            blockEntity.isFirst = false;
            TouhouNotaUtils.playAt(world, pos, blockEntity.getSelect());
            return;
        }

        if (hasRedstone && songPlayer == null) {
            TouhouNotaUtils.playAt(world, pos, blockEntity.getSelect());
            return;
        }

        if (!hasRedstone && songPlayer != null && songPlayer.isPlaying()) {
            songPlayer.setPlaying(false);
            blockPos2SongPlayer.remove(pos.method_10063());
        }
    }

    @Override
    protected void method_11007(class_11372 view) {
        super.method_11007(view);
        view.method_71469("Select", this.select == null ? "" : this.select);
    }

    @Override
    protected void method_11014(class_11368 view) {
        super.method_11014(view);
        this.select = view.method_71428("Select","");
    }

    public int play() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty() || select == null || !filenames.contains(select)) {
            return -1;
        }

        if (this.field_11863 != null && !this.field_11863.field_9236) {
            TouhouNotaUtils.playAt(field_11863, field_11867, select);
        }
        return filenames.indexOf(select);
    }

    public int prev() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty()) return -1;

        int index = select == null ? 0 : filenames.indexOf(select);
        if (index == -1) index = 0;

        index = (index - 1 + filenames.size()) % filenames.size();
        this.select = filenames.get(index);
        this.method_5431();
        return index;
    }

    public int next() {
        List<String> filenames = this.getFilenames();
        if (filenames.isEmpty()) return -1;

        int index = select == null ? -1 : filenames.indexOf(select);
        index = (index + 1) % filenames.size();
        this.select = filenames.get(index);
        this.method_5431();
        return index;
    }


    public List<String> getFilenames() {
        return TouhouNotaUtils.getFileNames();
    }
}
